#include <sys/types.h>
#include <openssl/err.h>
#include <openssl/rand.h>
#include <err.h>
#include <stdio.h>
#include <sysexits.h>

#include "vncryptctl.h"

#define DEFAULT_KEY_LENGTH (256 / 8)

struct aes_cbc_keydata {
	u_int32_t iv[4];		/* IV */
	size_t len;			/* key length, variable */
	char key[0];
};

static check_params_t check_aes_cbc;
static generate_t generate_aes_cbc;

static struct alg_params aes_cbc_params = {
	ALG_AES_CBC,
	"aes-cbc",
	"rijndael (AES) in single-key CBC mode",
	check_aes_cbc,
	generate_aes_cbc,
	NULL,
	NULL,
	NULL
};

static void
check_aes_cbc(u_long *key_length, u_long *fake) {
	if (*key_length == 0)
		*key_length = DEFAULT_KEY_LENGTH;

	if (*key_length != (128 / 8) && 
	    *key_length != (192 / 8) &&
	    *key_length != (256 / 8))
		errx(EX_USAGE,
		     "invalid aes key length, must be 16, 24 or 32\n");

	if (*fake != 0)
		errx(EX_USAGE, "aes-cbc has no subkeys");
}

static void
generate_aes_cbc(char **pptr, size_t *psize, u_long key_length,
		u_long fake __unused) {
	struct aes_cbc_keydata *pkey;

	LOG(("generating %ld-bit aes_cbc key\n", key_length * 8));
	
	*psize = sizeof(struct aes_cbc_keydata) + key_length;

	*pptr = malloc(*psize);
	if (*pptr == NULL)
		err(EX_TEMPFAIL, "can't allocate %d bytes", *psize);

	pkey = (struct aes_cbc_keydata *)*pptr;
	pkey->len = key_length;

	if (RAND_bytes((u_char *)pkey->iv, sizeof(pkey->iv)) != 1)
		errx(EX_TEMPFAIL, "can't get %d random bytes: %s",
		     sizeof(pkey->iv),
		     ERR_error_string(ERR_get_error(), NULL));

	if (RAND_bytes((u_char *)(pkey + 1), key_length) != 1)
		errx(EX_TEMPFAIL, "can't get %ld random bytes: %s",
		     key_length,
		     ERR_error_string(ERR_get_error(), NULL));
}

void
register_aes_cbc(void) {
	register_alg(&aes_cbc_params);
}
