#include <sys/types.h>
#include <openssl/err.h>
#include <openssl/rand.h>
#include <err.h>
#include <stdio.h>
#include <sysexits.h>

#include "vncryptctl.h"

#define DEFAULT_KEY_LENGTH (256 / 8)

struct aes_multi_keydata {
	u_int32_t iv[4];		/* IV */
	size_t subkey_len;		/* block key length */
	size_t len;			/* key length, variable */
	char key[0];
};

static check_params_t check_aes_multi;
static generate_t generate_aes_multi;

static struct alg_params aes_multi_params = {
	ALG_AES_MULTI,
	"aes-multi",
	"rijndael (AES) in multi-key CBC mode",
	check_aes_multi,
	generate_aes_multi,
	NULL,
	NULL,
	NULL
};

static void
check_aes_multi(u_long *key_length, u_long *subkey_length) {
	if (*key_length == 0)
		*key_length = DEFAULT_KEY_LENGTH;
	if (*subkey_length == 0)
		*subkey_length = DEFAULT_KEY_LENGTH;

	if (*key_length != (128 / 8) && 
	    *key_length != (192 / 8) &&
	    *key_length != (256 / 8))
		errx(EX_USAGE,
		     "invalid aes key length, must be 16, 24 or 32\n");
	if (*subkey_length != (128 / 8) && 
	    *subkey_length != (192 / 8) &&
	    *subkey_length != (256 / 8))
		errx(EX_USAGE,
		     "invalid aes-multi subkey length, must be 16, 24 or 32\n");
}

static void
generate_aes_multi(char **pptr, size_t *psize, u_long key_length,
		u_long subkey_length) {
	struct aes_multi_keydata *pkey;

	LOG(("generating %ld-bit aes_multi key, "
		"subkey length %ld bits\n", key_length * 8,
		subkey_length * 8));
	
	*psize = sizeof(struct aes_multi_keydata) + key_length;

	*pptr = malloc(*psize);
	if (*pptr == NULL)
		err(EX_TEMPFAIL, "can't allocate %d bytes", *psize);

	pkey = (struct aes_multi_keydata *)*pptr;
	pkey->subkey_len = subkey_length;
	pkey->len = key_length;

	if (RAND_bytes((u_char *)pkey->iv, sizeof(pkey->iv)) != 1)
		errx(EX_TEMPFAIL, "can't get %d random bytes: %s",
		     sizeof(pkey->iv),
		     ERR_error_string(ERR_get_error(), NULL));

	if (RAND_bytes((u_char *)(pkey + 1), key_length) != 1)
		errx(EX_TEMPFAIL, "can't get %ld random bytes: %s",
		     key_length,
		     ERR_error_string(ERR_get_error(), NULL));
}

void
register_aes_multi(void) {
	register_alg(&aes_multi_params);
}
