#include "vncryptctl.h"
#include <sys/queue.h>
#include <stdlib.h>

struct algrec {
	struct alg_params *params;
	SLIST_ENTRY(algrec) chain;
};

SLIST_HEAD(,algrec) alglist = SLIST_HEAD_INITIALIZER(dummy);

void
register_alg(struct alg_params *alginfo) {
	struct algrec *ptr = malloc(sizeof(struct algrec));

	ptr->params = alginfo;
	SLIST_INSERT_HEAD(&alglist, ptr, chain);
}

struct alg_params *
get_alg_by_id(u_int32_t id) {
	struct algrec *ptr;

	SLIST_FOREACH(ptr, &alglist, chain) {
		if (id == ptr->params->id)
			return ptr->params;
	}

	return NULL;
}

struct alg_params *
get_alg_by_name(const char *name) {
	struct algrec *ptr;

	SLIST_FOREACH(ptr, &alglist, chain) {
		if (!strcmp(name, ptr->params->name))
			return ptr->params;
	}

	return NULL;
}

struct alg_params *
get_first_alg() {
	struct algrec *ptr = SLIST_FIRST(&alglist);

	return (ptr)?(ptr->params):(NULL);
}

const char **
get_alg_names() {
	struct algrec *ptr;
	const char **names;
	int count = 0;

	SLIST_FOREACH(ptr, &alglist, chain)
		count++;

	names = malloc(sizeof(char *) * (count + 1));
	names[count] = NULL;
	count = 0;

	SLIST_FOREACH(ptr, &alglist, chain) {
		names[count++] = ptr->params->name;
	}

	return names;
}

const char **
get_alg_comments() {
	struct algrec *ptr;
	const char **comments;
	int count = 0;

	SLIST_FOREACH(ptr, &alglist, chain)
		count++;

	comments = malloc(sizeof(char *) * (count + 1));
	comments[count] = NULL;
	count = 0;

	SLIST_FOREACH(ptr, &alglist, chain) {
		comments[count++] = ptr->params->comment;
	}

	return comments;
}
