#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <openssl/blowfish.h>
#include <openssl/hmac.h>
#include <err.h>
#include <fcntl.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sysexits.h>
#include <unistd.h>

#include "vncryptctl.h"

static __dead2 void
show_help(void) {
	printf("Usage: vncryptctl enable [options] device keyfile\n");
	printf("Valid options:\n"
	       "\t-h\tthis help\n"
	       "\t-t timeout\tinactivity timeout\n");
	exit(0);
}

void
enable_encryption(int argc, char **argv) {
	struct vncr_generic_params *params;
	size_t keydata_len;
	u_int algid;
	u_long timeout = 0;
	int ch;
	int fd;

	optreset = 1;
	optind = 0;
	
	/* parsing options */
	
	while ((ch = getopt(argc, argv, "ht:")) != -1)
		switch (ch) {
		{
			char *lastchar;
		case 't':
			if (timeout != 0)
				errx(EX_USAGE, "timeout already selected");
			timeout = strtoul(optarg, &lastchar, 10);
			if (*lastchar != '\0' || timeout == ULONG_MAX)
				errx(EX_USAGE, "invalid timeout `%s'", optarg);
			break;
		}
		case 'h': /* FALL-THROUGH */
		default:
			show_help();
		}

	argc -= optind;
	argv += optind;

	if (argc != 2) {
		warnx("keyfile or device not specified");
		show_help();
	}

	/* loading keyfile */

	read_keyfile(argv[1], &params, &keydata_len, &algid);

	/* setting options */

	switch (algid) {
	case ALG_BF_CBC:
		params->vncr_algid = VNCRYPT_BF_CBC_ID;
		break;
	case ALG_AES_CBC:
		params->vncr_algid = VNCRYPT_AES_CBC_ID;
		break;
	case ALG_AES_MULTI:
		params->vncr_algid = VNCRYPT_AES_MULTI_ID;
		break;
	default:
		errx(EX_SOFTWARE, "internal error: unknown algorithm");
	}

	params->vncr_timeout = timeout;
	if (timeout)
		params->vncr_timeout_action = VNCRYPT_TO_DETACH;
	else
		params->vncr_timeout_action = VNCRYPT_TO_NONE;

	LOG(("enabling crypto with alg %d, timeout %ld\n", params->vncr_algid,
	     params->vncr_timeout));
	
	/* working with device */

	fd = open(argv[0], O_RDWR);
	if (fd == -1)
		err(EX_NOINPUT, "can't open device node `%s'", argv[0]);

	if (ioctl(fd, VNIOCCRYPT, &params) != 0)
		err(EX_OSERR, "can't enable encryption");

	close(fd);

	/* cleaning */

	bzero(params, sizeof(*params) + keydata_len);
	free(params);
}
