#include <sys/types.h>
#include <err.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <sysexits.h>
#include <unistd.h>

#include "vncryptctl.h"

int verbose = 0;
int passfd = -1;

static __dead2 void
show_help(void) {
	printf("Usage: vncryptctl [global-options] "
	       "command [command-options]\n");
	printf("Global options:\n"
	       "\t-h\tthis help\n"
	       "\t-v\tverbose mode\n"
	       "\t-p fd\tread passphrase from this descriptor\n");
	printf("Commands:\n"
	       "\tmakekey\tgenerate disk key\n"
	       "\tenable\tenable encryption on device\n");

	printf("Use vncryptctl command -h for command-specific help\n");

	exit(0);
}

int
main(int argc, char **argv) {
	int ch;

	register_bf_cbc();
	register_aes_cbc();
	register_aes_multi();
	
	if (argc == 1)
		show_help();
	
	/* parsing global options */
	while ((ch = getopt(argc, argv, "hvp:")) != -1)
		switch (ch) {
		case 'v':
			verbose++;
			break;
		{
			char *lastchar;
		case 'p':
			if (passfd != -1)
				errx(EX_USAGE,
				     "passphrase fd already selected");
			passfd = strtol(optarg, &lastchar, 10);
			if (*lastchar != '\0' || passfd == LONG_MAX
			    || passfd < 0)
				errx(EX_USAGE, "invalid fd `%s'", optarg);
			break;
		}
		case 'h': /* FALL-THROUGH */
		default:
			show_help();
		}
	argc -= optind;
	argv += optind;

	/* parsing command */
	if (argc == 0) {
		warnx("command not specified");
		show_help();
	}

	if (strcmp(argv[0], "makekey") == 0) {
		/* key generation */
		argv++;
		argc--;
		generate_key(argc, argv);
	} else if (strcmp(argv[0], "enable") == 0) {
		argv++;
		argc--;
		enable_encryption(argc, argv);
	} else {
		warnx("invalid command -- %s", argv[0]);
		show_help();
	}

	return 0;
}
