#ifndef VNCRYPTCTL_H
#define VNCRYPTCTL_H

#include <sys/types.h>
#include <openssl/md5.h>
#include <dev/vncrypt/vn_crypt.h>

extern int verbose;
extern int passfd;

#define LOG(x) if (verbose) { printf x; }

#define ALG_BF_CBC 1
#define ALG_AES_CBC 2
#define ALG_AES_MULTI 3

/* master key length (256 bit) */
#define MASTER_KEY_LEN 32

/* keyfile signature (VCKF) */
#define KEYFILE_SIG htonl(0x56434b46)

struct open_header {
	u_int32_t sig;
	u_int32_t keydata_len;
	u_int8_t mac[MD5_LBLOCK];
};

struct secret_header {
	u_int32_t alg_id;
};

/* algorithm-specific calls */

typedef void check_params_t(u_long *, u_long *);
typedef void generate_t(char **, size_t *, u_long, u_long);
typedef void *create_context_t(const void *, size_t);
typedef void clear_context_t(void *);
typedef void decrypt_block_t(void *, u_long, void *, size_t);

struct alg_params {
	u_int32_t id;
	const char *name;
	const char *comment;
	check_params_t *check_func;
	generate_t *generate_func;
	create_context_t *create_ctx_func;
	clear_context_t *clear_ctx_func;
	decrypt_block_t *decrypt_func;
};

void register_alg(struct alg_params *);
struct alg_params *get_alg_by_id(u_int32_t);
struct alg_params *get_alg_by_name(const char *);
struct alg_params *get_first_alg(void);
const char ** get_alg_names(void);
const char ** get_alg_comments(void);

/* generic calls */

void generate_key(int, char **);
void enable_encryption(int, char **);

/* utility */

void get_key_from_passphrase(void *, size_t);
void read_keyfile(const char *, struct vncr_generic_params **, size_t *,
		u_int *);

/* algs initialization */

void register_bf_cbc(void);
void register_aes_cbc(void);
void register_aes_multi(void);

#endif
